﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Input;

using Listener_Server_Example.Commands;
using System.Collections.ObjectModel;
using Unitronics.ComDriver;
using System.Windows.Threading;
using System.ComponentModel;
using System.Windows.Data;

namespace Listener_Server_Example.ViewModels
{
    public class MainViewModel : ViewModelBase
    {
        private ObservableCollection<ClientViewModel> clients = new ObservableCollection<ClientViewModel>();
        private DelegateCommand exitCommand;
        private DelegateCommand listenCommand;
        private DelegateCommand disconnectCommand;
        private bool listening = false;
        private ListenerServer listenerServer;
        private const int LOCAL_PORT = 20275;
        private string status;
        private Dispatcher dispatcher;

        #region Constructor

        public MainViewModel()
        {
            Status = "Ready";
            dispatcher = Dispatcher.CurrentDispatcher;
        }

        #endregion

        public ObservableCollection<ClientViewModel> Clients
        {
            get
            {
                return clients;
            }
            internal set
            {
                clients = value;
                OnPropertyChanged("Clients");
            }
        }

        public string Status
        {
            get
            {
                return status;
            }
            internal set
            {
                status = value;
                OnPropertyChanged("Status");
            }
        }

        public ICommand ExitCommand
        {
            get
            {
                if (exitCommand == null)
                {
                    exitCommand = new DelegateCommand(Exit);
                }
                return exitCommand;
            }
        }

        private void Exit()
        {
            Application.Current.Shutdown();
        }

        public ICommand ListenCommand
        {
            get
            {
                if (listenCommand == null)
                {
                    listenCommand = new DelegateCommand(Listen, canListen);
                }
                return listenCommand;
            }
        }

        private bool canListen()
        {
            return !listening;
        }

        private void Listen()
        {
            try
            {
                PLCFactory.GetChannel(LOCAL_PORT, out listenerServer);

                if (listenerServer == null)
                    listenerServer = new ListenerServer(LOCAL_PORT, 3, 3000);

                listenerServer.OnConnectionAccepted += new ListenerServer.ConnectionAcceptedDelegate(listenerServer_OnConnectionAccepted);
                listenerServer.OnConnectionStatusChanged += new ListenerServer.ConnectionStatusChangedDelegate(listenerServer_OnConnectionStatusChanged);

                PLCFactory.GetPLC(listenerServer);
                listening = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        void listenerServer_OnConnectionStatusChanged(EthernetListener.ConnectionStatus connectionStatus)
        {
            // You can also use a timer to scan the status every, lets say, 1 second.

            updateStatus(connectionStatus);
        }

        private void updateStatus(EthernetListener.ConnectionStatus connectionStatus)
        {
            string text = "";
            switch (connectionStatus)
            {
                case EthernetListener.ConnectionStatus.Listening:
                    text = "Listening, Connected Clients: " + getNumberOfClients().ToString();
                    break;
                case EthernetListener.ConnectionStatus.Disconnected:
                    text = "Disconnected, Connected Clients: " + getNumberOfClients().ToString();
                    break;
            }

            Status = text;
        }

        void listenerServer_OnConnectionAccepted(PLC plcFromListener)
        {
            // This event is not raised from the GUIs thread. Therefore we need to invoke it on the
            // Main thread using the dispatcher. It is important to create the viewmodels in the GUI threa
            // cause we want the dispatcher of the viewmodels to be associated with the right thread.
            Action action = delegate()
            {
                ClientViewModel clientViewModel = new ClientViewModel(plcFromListener);
                clientViewModel.OnConnectionClosed += new EventHandler(clientViewModel_OnConnectionClosed);
                clients.Add(clientViewModel);
                updateStatus(listenerServer.Status);
                refreshIndexes();
            };
            dispatcher.Invoke(action, null);
        }

        void clientViewModel_OnConnectionClosed(object sender, EventArgs e)
        {
            try
            {
                Action action = delegate()
                {
                    clients.Remove(sender as ClientViewModel);
                    updateStatus(listenerServer.Status);
                    refreshIndexes();
                };
                dispatcher.Invoke(action, null);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void refreshIndexes()
        {
            ICollectionView view = CollectionViewSource.GetDefaultView(Clients) as ListCollectionView;
            view.Refresh();
        }

        public ICommand DisconnectCommand
        {
            get
            {
                if (disconnectCommand == null)
                {
                    disconnectCommand = new DelegateCommand(Disconnect, canDisconnect);
                }
                return disconnectCommand;
            }
        }

        private bool canDisconnect()
        {
            return listening;
        }

        private void Disconnect()
        {
            if (listenerServer != null)
            {
                try
                {
                    listenerServer.Disconnect();
                    listenerServer.OnConnectionAccepted -= new ListenerServer.ConnectionAcceptedDelegate(listenerServer_OnConnectionAccepted);
                    listening = false;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private int getNumberOfClients()
        {
            int result = 0;
            Details detail = ListenerClientsInfo.GetDetails(LOCAL_PORT);
            if (detail != null)
            {
                result = detail.Count;
            }
            return result;
        }
    }
}
